@@ -1,4 +1,4 @@
-#Copyright ReportLab Europe Ltd. 2000-2016
+#Copyright ReportLab Europe Ltd. 2000-2017
 #see license.txt for license details
 __version__ = '$Id$'
 __doc__="""TrueType font support
@@ -54,10 +54,10 @@ Canvas and TextObject have special support for dynamic fonts.
 from struct import pack, unpack, error as structError
 from reportlab.lib.utils import getBytesIO, isPy3, bytestr, isUnicode, char2int
 from reportlab.pdfbase import pdfmetrics, pdfdoc
-from reportlab import rl_config
+from reportlab import rl_config, xrange, ascii
 from reportlab.lib.rl_accel import hex32, add32, calcChecksum, instanceStringWidthTTF
 from collections import namedtuple
-import time
+import os, time
 
 class TTFError(pdfdoc.PDFError):
     "TrueType font exception"
@@ -527,7 +527,10 @@ class TTFontFile(TTFontParser):
         self.uniqueFontID = names[3] or psName
 
         # head - Font header table
-        self.seek_table("head")
+        try:
+            self.seek_table("head")
+        except:
+            raise TTFError('head table not found ttf name=%s' % self.name)
         ver_maj, ver_min = self.read_ushort(), self.read_ushort()
         if ver_maj != 1:
             raise TTFError('Unknown head table version %d.%04x' % (ver_maj, ver_min))
@@ -552,6 +555,7 @@ class TTFontFile(TTFontParser):
 
         # OS/2 - OS/2 and Windows metrics table
         # (needs data from head table)
+        subsettingAllowed = True
         if "OS/2" in self.table:
             self.seek_table("OS/2")
             version = self.read_ushort()
@@ -559,8 +563,8 @@ class TTFontFile(TTFontParser):
             usWeightClass = self.read_ushort()
             self.skip(2)
             fsType = self.read_ushort()
-            if fsType == 0x0002 or (fsType & 0x0300) != 0:
-                raise TTFError('Font does not allow subsetting/embedding (%04X)' % fsType)
+            if fsType==0x0002 or (fsType & 0x0300):
+                subsettingAllowed = os.path.basename(self.filename) not in rl_config.allowTTFSubsetting
             self.skip(58)   #11*2 + 10 + 4*4 + 4 + 3*2
             sTypoAscender = self.read_short()
             sTypoDescender = self.read_short()
@@ -630,6 +634,13 @@ class TTFontFile(TTFontParser):
         if ver_maj != 1:
             raise TTFError('Unknown maxp table version %d.%04x' % (ver_maj, ver_min))
         self.numGlyphs = numGlyphs = self.read_ushort()
+        if not subsettingAllowed:
+            if self.numGlyphs>0xFF:
+                raise TTFError('Font does not allow subsetting/embedding (%04X)' % fsType)
+            else:
+                self._full_font = True
+        else:
+            self._full_font = False
 
         if not charInfo:
             self.charToGlyph = None
@@ -1093,11 +1104,22 @@ class TTFont:
     """
     class State:
         namePrefix = 'F'
-        def __init__(self,asciiReadable=None):
-            self.assignments = {}
+        def __init__(self,asciiReadable=None,ttf=None):
+            A = self.assignments = {}
             self.nextCode = 0
             self.internalName = None
             self.frozen = 0
+            if getattr(getattr(ttf,'face',None),'_full_font',None):
+                C = set(self.charToGlyph.keys())
+                for n in xrange(256):
+                    if n in C:
+                        A[n] = n
+                        C.remove(n)
+                for n in C:
+                    A[n] = n
+                self.subsets = [[n for n in A]]
+                self.frozen = True
+                return
 
             if asciiReadable is None:
                 asciiReadable = rl_config.ttfAsciiReadable
@@ -1109,11 +1131,11 @@ class TTFont:
                 subset0 = list(xrange(128))
                 self.subsets = [subset0]
                 for n in subset0:
-                    self.assignments[n] = n
+                    A[n] = n
                 self.nextCode = 128
             else:
                 self.subsets = [[32]*33]
-                self.assignments[32] = 32
+                A[32] = 32
 
     _multiByte = 1      # We want our own stringwidth
     _dynamicFont = 1    # We want dynamic subsetting
@@ -1143,7 +1165,7 @@ class TTFont:
         try:
             state = self.state[doc]
         except KeyError:
-            state = self.state[doc] = TTFont.State(asciiReadable)
+            state = self.state[doc] = TTFont.State(asciiReadable,self)
             if namePrefix is not None:
                 state.namePrefix = namePrefix
         return state
@@ -1155,7 +1177,7 @@ class TTFont:
         subsets when building different documents at the same time."""
         asciiReadable = self._asciiReadable
         try: state = self.state[doc]
-        except KeyError: state = self.state[doc] = TTFont.State(asciiReadable)
+        except KeyError: state = self.state[doc] = TTFont.State(asciiReadable,self)
         curSet = -1
         cur = []
         results = []
